(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('ramda'), require('prop-types'), require('ramda-extension'), require('@redux-tools/utils')) :
typeof define === 'function' && define.amd ? define(['exports', 'react', 'ramda', 'prop-types', 'ramda-extension', '@redux-tools/utils'], factory) :
(factory((global.ReduxToolsInjectorsReact = {}),global.React,global.R,global.PropTypes,global.R_,global.ReduxToolsUtils));
}(this, (function (exports,React,ramda,PropTypes,ramdaExtension,utils) { 'use strict';

var React__default = 'default' in React ? React['default'] : React;
PropTypes = PropTypes && PropTypes.hasOwnProperty('default') ? PropTypes['default'] : PropTypes;

var InjectorContext = React.createContext({
  store: null,
  withNamespace: ramda.identity
});

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var counter = 0;

withSuffixing.resetCounter = function () {
  return counter = 0;
};

var suffix = function suffix(id) {
  return function (value) {
    return ramda.join('@', [value, id]);
  };
};
/**
 * This HOC is necessary because if a container is remounted for any reason,
 * the `componentWillUnmount` lifecycle hook is invoked AFTER the constructor of the new
 * element. This would cause the reducers to be removed and not injected back.
 *
 * `->` is constructor of the second instance and `=>` is CWU of the first one
 * BEFORE: [reducer] -> [reducer] => [], no reducer survives the remounting process
 * AFTER: [reducer1] -> [reducer1, reducer2] => [reducer2], the second reducer survives
 *
 * @param {React.Component} NextComponent component to wrap
 * @returns {React.Component} component with a bound `suffixKeys` prop
 */


function withSuffixing(NextComponent) {
  var WithSuffixing =
  /*#__PURE__*/
  function (_Component) {
    _inheritsLoose(WithSuffixing, _Component);

    function WithSuffixing() {
      var _this;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      _this = _Component.call.apply(_Component, [this].concat(args)) || this;
      _this.id = counter++;
      _this.suffixKeys = ramda.o(ramda.map(ramda.when(ramdaExtension.isPlainObject, function (object) {
        return _this.suffixKeys(object);
      })), ramdaExtension.mapKeys(suffix(_this.id)));
      return _this;
    }

    var _proto = WithSuffixing.prototype;

    _proto.render = function render() {
      return React__default.createElement(NextComponent, _extends({
        suffixKeys: this.suffixKeys
      }, this.props));
    };

    return WithSuffixing;
  }(React.Component);

  WithSuffixing.propTypes = {
    namespace: PropTypes.string
  };
  WithSuffixing.displayName = "WithSuffixing(" + utils.getDisplayName(NextComponent) + ")";
  return WithSuffixing;
}

function withInjectorContext(NextComponent) {
  var WithInjectorContext = function WithInjectorContext(props) {
    return React__default.createElement(InjectorContext.Consumer, null, function (_ref) {
      var _ref$withNamespace = _ref.withNamespace,
          withNamespace = _ref$withNamespace === void 0 ? ramda.identity : _ref$withNamespace,
          store = _ref.store;

      // NOTE: React's reconciliation process would otherwise think that we're rendering
      // two different components (because we would be creating a new one each render).
      // TODO: Allow changing the `withNamespace` prop.
      if (!WithInjectorContext.WrappedComponent) {
        WithInjectorContext.WrappedComponent = withNamespace(NextComponent);
      }

      return React__default.createElement(WithInjectorContext.WrappedComponent, _extends({
        store: store
      }, props));
    });
  }; // NOTE: We attach `WrappedComponent` to `WithInjectorContext` because of tests.


  WithInjectorContext.WrappedComponent = null;
  return WithInjectorContext;
}

var filterProps = ramda.omit(['suffixKeys', 'store']);
function makeInjector(inject, remove) {
  return function (injectables, _temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        persist = _ref.persist,
        global = _ref.global;

    return function (NextComponent) {
      var Injector =
      /*#__PURE__*/
      function (_Component) {
        _inheritsLoose(Injector, _Component);

        function Injector(props) {
          var _this;

          _this = _Component.call(this, props) || this; // TODO: Refactor to allow changes over time in namespace and injectables

          _this.injectables = props.suffixKeys(injectables);
          _this.namespace = global ? null : props.namespace;
          inject(props.store)(_this.injectables, _this.namespace);
          return _this;
        }

        var _proto = Injector.prototype;

        _proto.componentWillUnmount = function componentWillUnmount() {
          if (!persist) {
            remove(this.props.store)(ramda.keys(this.injectables), this.namespace);
          }
        };

        _proto.render = function render() {
          return React__default.createElement(NextComponent, filterProps(this.props));
        };

        return Injector;
      }(React.Component);

      Injector.propTypes = {
        namespace: PropTypes.string,
        store: PropTypes.object.isRequired,
        suffixKeys: PropTypes.func.isRequired
      };
      Injector.displayName = "Injector(" + utils.getDisplayName(NextComponent) + ")";
      return ramda.o(withSuffixing, withInjectorContext)(Injector);
    };
  };
}

var Provider = InjectorContext.Provider;

exports.Provider = Provider;
exports.makeInjector = makeInjector;
exports.withInjectorContext = withInjectorContext;
exports.withSuffixing = withSuffixing;

Object.defineProperty(exports, '__esModule', { value: true });

})));
